# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from fabric_cf.orchestrator.swagger_server.models.base_model_ import Model
from fabric_cf.orchestrator.swagger_server.models.sliver import Sliver  # noqa: F401,E501
from fabric_cf.orchestrator.swagger_server.models.status200_ok_paginated import Status200OkPaginated  # noqa: F401,E501
from fabric_cf.orchestrator.swagger_server import util


class Slivers(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, limit: int=None, offset: int=None, size: int=None, status: int=200, total: int=None, type: str=None, data: List[Sliver]=None):  # noqa: E501
        """Slivers - a model defined in Swagger

        :param limit: The limit of this Slivers.  # noqa: E501
        :type limit: int
        :param offset: The offset of this Slivers.  # noqa: E501
        :type offset: int
        :param size: The size of this Slivers.  # noqa: E501
        :type size: int
        :param status: The status of this Slivers.  # noqa: E501
        :type status: int
        :param total: The total of this Slivers.  # noqa: E501
        :type total: int
        :param type: The type of this Slivers.  # noqa: E501
        :type type: str
        :param data: The data of this Slivers.  # noqa: E501
        :type data: List[Sliver]
        """
        self.swagger_types = {
            'limit': int,
            'offset': int,
            'size': int,
            'status': int,
            'total': int,
            'type': str,
            'data': List[Sliver]
        }

        self.attribute_map = {
            'limit': 'limit',
            'offset': 'offset',
            'size': 'size',
            'status': 'status',
            'total': 'total',
            'type': 'type',
            'data': 'data'
        }
        self._limit = limit
        self._offset = offset
        self._size = size
        self._status = status
        self._total = total
        self._type = type
        self._data = data

    @classmethod
    def from_dict(cls, dikt) -> 'Slivers':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The slivers of this Slivers.  # noqa: E501
        :rtype: Slivers
        """
        return util.deserialize_model(dikt, cls)

    @property
    def limit(self) -> int:
        """Gets the limit of this Slivers.


        :return: The limit of this Slivers.
        :rtype: int
        """
        return self._limit

    @limit.setter
    def limit(self, limit: int):
        """Sets the limit of this Slivers.


        :param limit: The limit of this Slivers.
        :type limit: int
        """

        self._limit = limit

    @property
    def offset(self) -> int:
        """Gets the offset of this Slivers.


        :return: The offset of this Slivers.
        :rtype: int
        """
        return self._offset

    @offset.setter
    def offset(self, offset: int):
        """Sets the offset of this Slivers.


        :param offset: The offset of this Slivers.
        :type offset: int
        """

        self._offset = offset

    @property
    def size(self) -> int:
        """Gets the size of this Slivers.


        :return: The size of this Slivers.
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size: int):
        """Sets the size of this Slivers.


        :param size: The size of this Slivers.
        :type size: int
        """

        self._size = size

    @property
    def status(self) -> int:
        """Gets the status of this Slivers.


        :return: The status of this Slivers.
        :rtype: int
        """
        return self._status

    @status.setter
    def status(self, status: int):
        """Sets the status of this Slivers.


        :param status: The status of this Slivers.
        :type status: int
        """

        self._status = status

    @property
    def total(self) -> int:
        """Gets the total of this Slivers.


        :return: The total of this Slivers.
        :rtype: int
        """
        return self._total

    @total.setter
    def total(self, total: int):
        """Sets the total of this Slivers.


        :param total: The total of this Slivers.
        :type total: int
        """

        self._total = total

    @property
    def type(self) -> str:
        """Gets the type of this Slivers.


        :return: The type of this Slivers.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type: str):
        """Sets the type of this Slivers.


        :param type: The type of this Slivers.
        :type type: str
        """

        self._type = type

    @property
    def data(self) -> List[Sliver]:
        """Gets the data of this Slivers.


        :return: The data of this Slivers.
        :rtype: List[Sliver]
        """
        return self._data

    @data.setter
    def data(self, data: List[Sliver]):
        """Sets the data of this Slivers.


        :param data: The data of this Slivers.
        :type data: List[Sliver]
        """

        self._data = data



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from __future__ import annotations

import logging
import threading
import traceback
from typing import TYPE_CHECKING

from fabric_mb.message_bus.producer import AvroProducerApi

from fabric_cf.actor.core.kernel.retry_rpc import RetryRPC
from fabric_cf.actor.core.util.rpc_exception import RPCException
from fabric_cf.actor.core.kernel.failed_rpc import FailedRPC
from fabric_cf.actor.core.kernel.failed_rpc_event import FailedRPCEvent

if TYPE_CHECKING:
    from fabric_cf.actor.core.kernel.rpc_request import RPCRequest


class RPCExecutor:
    """
    Execute an RPC
    """
    @staticmethod
    def post_exception(request: RPCRequest, e: RPCException):
        """
        Handle any exception raised during processing
        """
        logger = request.actor.get_logger()
        try:
            logger.error("An error occurred while performing RPC. Error type={} {}".format(e.get_error_type(), e))
            from fabric_cf.actor.core.kernel.rpc_manager_singleton import RPCManagerSingleton
            RPCManagerSingleton.get().remove_pending_request(request.request.get_message_id())
            failed = FailedRPC(e=e, request=request)
            request.actor.queue_event(FailedRPCEvent(actor=request.actor, failed=failed))
        except Exception as e:
            logger.error("postException failed = {}".format(e))

    @staticmethod
    def run(request: RPCRequest, producer: AvroProducerApi):
        """
        Execute RPC
        """
        logger = request.actor.get_logger()
        logger.debug(f"Performing RPC: type={request.request.get_type()} to:{request.proxy.get_name()}")
        try:
            request.proxy.execute(request=request.request, producer=producer)
            if request.handler is None:
                if request.timer is not None:
                    logger.debug("Canceling the timer: {}".format(request.timer))
                request.cancel_timer()
        except RPCException as e:
            RPCExecutor.post_exception(request=request, e=e)
        finally:
            logger.debug("Completed RPC: type= {} to: {}".format(request.request.get_type(),
                                                                 request.proxy.get_name()))
            from fabric_cf.actor.core.kernel.rpc_manager_singleton import RPCManagerSingleton
            RPCManagerSingleton.get().de_queued()

    @staticmethod
    def retry(request: RetryRPC, producer: AvroProducerApi, logger: logging.Logger):
        """
        Execute RPC
        """
        logger.debug(f"Performing RPC: type={request.message().name} to:{request.topic()}")
        try:
            producer.produce(topic=request.topic(), record=request.message())
        except RPCException as e:
            logger.error(f"Exception occurred in Retry RPC: {e}")
            logger.error(traceback.format_exc())
        finally:
            logger.debug(f"Completed RPC: type={request.message().name} to:{request.topic()}")


# coding: utf-8
#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from setuptools import setup, find_packages
from fabric_cf import __VERSION__

NAME = "fabric-cf"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    requirements = fh.read()

setup(
    name=NAME,
    version=__VERSION__,
    description="Fabric Control Framework",
    author="Komal Thareja, Ilya Baldin",
    author_email="kthare10@renci.org, ibaldin@renci.org",
    url="https://github.com/fabric-testbed/ControlFramework",
    keywords=["Swagger", "Fabric Control Framework"],
    install_requires=requirements,
    setup_requires=requirements,
    packages=find_packages(),
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.9'
)



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)

GET_METHOD = 'get'
POST_METHOD = 'post'
PUT_METHOD = 'put'
DELETE_METHOD = 'delete'

RESOURCES_PATH = '/resources'
PORTAL_RESOURCES_PATH = '/portalresources'

SLICES_CREATE_PATH = '/slices/create'
SLICES_MODIFY_PATH = '/slices/modify/{slice_id}'
SLICES_MODIFY_ACCEPT_PATH = '/slices/modify/{slice_id}/accept'
SLICES_DELETE_PATH = '/slices/delete/{slice_id}'
SLICES_GET_PATH = '/slices'
SLICES_GET_SLICE_ID_PATH = '/slices/{slice_id}'
SLICES_RENEW_PATH = '/slices/renew/{slice_id}'
SLICE_STATUS_SLICE_ID_PATH = '/slices/status/{slice_id}'


SLIVERS_GET_PATH = '/slivers'
SLIVERS_GET_SLIVER_ID_PATH = '/slivers/{sliver_id}'
SLIVERS_STATUS_SLIVER_ID_PATH = '/slivers/status/{sliver_id}'

VERSIONS_PATH = '/version'
